'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = require('fs');
var process$1 = require('process');
var path = require('path');
var path__default = _interopDefault(path);
var msalCommon = require('@azure/msal-common');
var keytar = require('keytar');

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
const Constants = {
  /**
   * An existing file was the target of an operation that required that the target not exist
   */
  EEXIST_ERROR: "EEXIST",

  /**
   * No such file or directory: Commonly raised by fs operations to indicate that a component
   * of the specified pathname does not exist. No entity (file or directory) could be found
   * by the given path
   */
  ENOENT_ERROR: "ENOENT",

  /**
   * Operation not permitted. An attempt was made to perform an operation that requires
   * elevated privileges.
   */
  EPERM_ERROR: "EPERM",

  /**
   * Default service name for using MSAL Keytar
   */
  DEFAULT_SERVICE_NAME: "msal-node-extensions",

  /**
   * Test data used to verify underlying persistence mechanism
   */
  PERSISTENCE_TEST_DATA: "Dummy data to verify underlying persistence mechanism",

  /**
   * This is the value of a the guid if the process is being ran by the root user
   */
  LINUX_ROOT_USER_GUID: 0,

  /**
   * List of environment variables
   */
  ENVIRONMENT: {
    HOME: "HOME",
    LOGNAME: "LOGNAME",
    USER: "USER",
    LNAME: "LNAME",
    USERNAME: "USERNAME",
    PLATFORM: "platform",
    LOCAL_APPLICATION_DATA: "LOCALAPPDATA"
  },
  // Name of the default cache file
  DEFAULT_CACHE_FILE_NAME: "cache.json"
};
var Platform;

(function (Platform) {
  Platform["WINDOWS"] = "win32";
  Platform["LINUX"] = "linux";
  Platform["MACOS"] = "darwin";
})(Platform || (Platform = {}));

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

/**
 * Error thrown when trying to write MSAL cache to persistence.
 */
class PersistenceError extends Error {
  constructor(errorCode, errorMessage) {
    const errorString = errorMessage ? `${errorCode}: ${errorMessage}` : errorCode;
    super(errorString);
    Object.setPrototypeOf(this, PersistenceError.prototype);
    this.errorCode = errorCode;
    this.errorMessage = errorMessage;
    this.name = "PersistenceError";
  }
  /**
   * Error thrown when trying to access the file system.
   */


  static createFileSystemError(errorCode, errorMessage) {
    return new PersistenceError(errorCode, errorMessage);
  }
  /**
   * Error thrown when trying to write, load, or delete data from secret service on linux.
   * Libsecret is used to access secret service.
   */


  static createLibSecretError(errorMessage) {
    return new PersistenceError("GnomeKeyringError", errorMessage);
  }
  /**
   * Error thrown when trying to write, load, or delete data from keychain on macOs.
   */


  static createKeychainPersistenceError(errorMessage) {
    return new PersistenceError("KeychainError", errorMessage);
  }
  /**
   * Error thrown when trying to encrypt or decrypt data using DPAPI on Windows.
   */


  static createFilePersistenceWithDPAPIError(errorMessage) {
    return new PersistenceError("DPAPIEncryptedFileError", errorMessage);
  }
  /**
   * Error thrown when using the cross platform lock.
   */


  static createCrossPlatformLockError(errorMessage) {
    return new PersistenceError("CrossPlatformLockError", errorMessage);
  }
  /**
   * Throw cache persistence error
   *
   * @param errorMessage string
   * @returns PersistenceError
   */


  static createCachePersistenceError(errorMessage) {
    return new PersistenceError("CachePersistenceError", errorMessage);
  }
  /**
   * Throw unsupported error
   *
   * @param errorMessage string
   * @returns PersistenceError
   */


  static createNotSupportedError(errorMessage) {
    return new PersistenceError("NotSupportedError", errorMessage);
  }
  /**
   * Throw persistence not verified error
   *
   * @param errorMessage string
   * @returns PersistenceError
   */


  static createPersistenceNotVerifiedError(errorMessage) {
    return new PersistenceError("PersistenceNotVerifiedError", errorMessage);
  }
  /**
   * Throw persistence creation validation error
   *
   * @param errorMessage string
   * @returns PersistenceError
   */


  static createPersistenceNotValidatedError(errorMessage) {
    return new PersistenceError("PersistenceNotValidatedError", errorMessage);
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Cross-process lock that works on all platforms.
 */

class CrossPlatformLock {
  constructor(lockFilePath, logger, lockOptions) {
    this.lockFilePath = lockFilePath;
    this.retryNumber = lockOptions ? lockOptions.retryNumber : 500;
    this.retryDelay = lockOptions ? lockOptions.retryDelay : 100;
    this.logger = logger;
  }
  /**
   * Locks cache from read or writes by creating file with same path and name as
   * cache file but with .lockfile extension. If another process has already created
   * the lockfile, will back off and retry based on configuration settings set by CrossPlatformLockOptions
   */


  async lock() {
    for (let tryCount = 0; tryCount < this.retryNumber; tryCount++) {
      try {
        this.logger.info(`Pid ${process$1.pid} trying to acquire lock`);
        this.lockFileHandle = await fs.promises.open(this.lockFilePath, "wx+");
        this.logger.info(`Pid ${process$1.pid} acquired lock`);
        await this.lockFileHandle.write(process$1.pid.toString());
        return;
      } catch (err) {
        if (err.code === Constants.EEXIST_ERROR || err.code === Constants.EPERM_ERROR) {
          this.logger.info(err);
          await this.sleep(this.retryDelay);
        } else {
          this.logger.error(`${process$1.pid} was not able to acquire lock. Ran into error: ${err.message}`);
          throw PersistenceError.createCrossPlatformLockError(err.message);
        }
      }
    }

    this.logger.error(`${process$1.pid} was not able to acquire lock. Exceeded amount of retries set in the options`);
    throw PersistenceError.createCrossPlatformLockError("Not able to acquire lock. Exceeded amount of retries set in options");
  }
  /**
   * unlocks cache file by deleting .lockfile.
   */


  async unlock() {
    try {
      if (this.lockFileHandle) {
        // if we have a file handle to the .lockfile, delete lock file
        await fs.promises.unlink(this.lockFilePath);
        await this.lockFileHandle.close();
        this.logger.info("lockfile deleted");
      } else {
        this.logger.warning("lockfile handle does not exist, so lockfile could not be deleted");
      }
    } catch (err) {
      if (err.code === Constants.ENOENT_ERROR) {
        this.logger.info("Tried to unlock but lockfile does not exist");
      } else {
        this.logger.error(`${process$1.pid} was not able to release lock. Ran into error: ${err.message}`);
        throw PersistenceError.createCrossPlatformLockError(err.message);
      }
    }
  }

  sleep(ms) {
    return new Promise(resolve => {
      setTimeout(resolve, ms);
    });
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * MSAL cache plugin which enables callers to write the MSAL cache to disk on Windows,
 * macOs, and Linux.
 *
 * - Persistence can be one of:
 * - FilePersistence: Writes and reads from an unencrypted file. Can be used on Windows,
 * macOs, or Linux.
 * - FilePersistenceWithDataProtection: Used on Windows, writes and reads from file encrypted
 * with windows dpapi-addon.
 * - KeychainPersistence: Used on macOs, writes and reads from keychain.
 * - LibSecretPersistence: Used on linux, writes and reads from secret service API. Requires
 * libsecret be installed.
 */

class PersistenceCachePlugin {
  constructor(persistence, lockOptions) {
    this.persistence = persistence; // initialize logger

    this.logger = persistence.getLogger(); // create file lock

    this.lockFilePath = `${this.persistence.getFilePath()}.lockfile`;
    this.crossPlatformLock = new CrossPlatformLock(this.lockFilePath, this.logger, lockOptions); // initialize default values

    this.lastSync = 0;
    this.currentCache = null;
  }
  /**
   * Reads from storage and saves an in-memory copy. If persistence has not been updated
   * since last time data was read, in memory copy is used.
   *
   * If cacheContext.cacheHasChanged === true, then file lock is created and not deleted until
   * afterCacheAccess() is called, to prevent the cache file from changing in between
   * beforeCacheAccess() and afterCacheAccess().
   */


  async beforeCacheAccess(cacheContext) {
    this.logger.info("Executing before cache access");
    const reloadNecessary = await this.persistence.reloadNecessary(this.lastSync);

    if (!reloadNecessary && this.currentCache !== null) {
      if (cacheContext.cacheHasChanged) {
        this.logger.verbose("Cache context has changed");
        await this.crossPlatformLock.lock();
      }

      return;
    }

    try {
      this.logger.info(`Reload necessary. Last sync time: ${this.lastSync}`);
      await this.crossPlatformLock.lock();
      this.currentCache = await this.persistence.load();
      this.lastSync = new Date().getTime();
      cacheContext.tokenCache.deserialize(this.currentCache);
      this.logger.info(`Last sync time updated to: ${this.lastSync}`);
    } finally {
      if (!cacheContext.cacheHasChanged) {
        await this.crossPlatformLock.unlock();
        this.logger.info(`Pid ${process$1.pid} released lock`);
      } else {
        this.logger.info(`Pid ${process$1.pid} beforeCacheAccess did not release lock`);
      }
    }
  }
  /**
   * Writes to storage if MSAL in memory copy of cache has been changed.
   */


  async afterCacheAccess(cacheContext) {
    this.logger.info("Executing after cache access");

    try {
      if (cacheContext.cacheHasChanged) {
        this.logger.info("Msal in-memory cache has changed. Writing changes to persistence");
        this.currentCache = cacheContext.tokenCache.serialize();
        await this.persistence.save(this.currentCache);
      } else {
        this.logger.info("Msal in-memory cache has not changed. Did not write to persistence");
      }
    } finally {
      await this.crossPlatformLock.unlock();
      this.logger.info(`Pid ${process$1.pid} afterCacheAccess released lock`);
    }
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
class BasePersistence {
  async verifyPersistence() {
    // We are using a different location for the test to avoid overriding the functional cache
    const persistenceValidator = await this.createForPersistenceValidation();

    try {
      await persistenceValidator.save(Constants.PERSISTENCE_TEST_DATA);
      const retrievedDummyData = await persistenceValidator.load();

      if (!retrievedDummyData) {
        throw PersistenceError.createCachePersistenceError("Persistence check failed. Data was written but it could not be read. " + "Possible cause: on Linux, LibSecret is installed but D-Bus isn't running because it cannot be started over SSH.");
      }

      if (retrievedDummyData !== Constants.PERSISTENCE_TEST_DATA) {
        throw PersistenceError.createCachePersistenceError(`Persistence check failed. Data written ${Constants.PERSISTENCE_TEST_DATA} is different from data read ${retrievedDummyData}`);
      }

      await persistenceValidator.delete();
      return true;
    } catch (e) {
      throw PersistenceError.createCachePersistenceError(`Verifing persistence failed with the error: ${e}`);
    }
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Reads and writes data to file specified by file location. File contents are not
 * encrypted.
 *
 * If file or directory has not been created, it FilePersistence.create() will create
 * file and any directories in the path recursively.
 */

class FilePersistence extends BasePersistence {
  static async create(fileLocation, loggerOptions) {
    const filePersistence = new FilePersistence();
    filePersistence.filePath = fileLocation;
    filePersistence.logger = new msalCommon.Logger(loggerOptions || FilePersistence.createDefaultLoggerOptions());
    await filePersistence.createCacheFile();
    return filePersistence;
  }

  async save(contents) {
    try {
      await fs.promises.writeFile(this.getFilePath(), contents, "utf-8");
    } catch (err) {
      throw PersistenceError.createFileSystemError(err.code, err.message);
    }
  }

  async saveBuffer(contents) {
    try {
      await fs.promises.writeFile(this.getFilePath(), contents);
    } catch (err) {
      throw PersistenceError.createFileSystemError(err.code, err.message);
    }
  }

  async load() {
    try {
      return await fs.promises.readFile(this.getFilePath(), "utf-8");
    } catch (err) {
      throw PersistenceError.createFileSystemError(err.code, err.message);
    }
  }

  async loadBuffer() {
    try {
      return await fs.promises.readFile(this.getFilePath());
    } catch (err) {
      throw PersistenceError.createFileSystemError(err.code, err.message);
    }
  }

  async delete() {
    try {
      await fs.promises.unlink(this.getFilePath());
      return true;
    } catch (err) {
      if (err.code === Constants.ENOENT_ERROR) {
        // file does not exist, so it was not deleted
        this.logger.warning("Cache file does not exist, so it could not be deleted");
        return false;
      }

      throw PersistenceError.createFileSystemError(err.code, err.message);
    }
  }

  getFilePath() {
    return this.filePath;
  }

  async reloadNecessary(lastSync) {
    return lastSync < (await this.timeLastModified());
  }

  getLogger() {
    return this.logger;
  }

  createForPersistenceValidation() {
    const testCacheFileLocation = `${path.dirname(this.filePath)}/test.cache`;
    return FilePersistence.create(testCacheFileLocation);
  }

  static createDefaultLoggerOptions() {
    return {
      loggerCallback: () => {// allow users to not set loggerCallback
      },
      piiLoggingEnabled: false,
      logLevel: msalCommon.LogLevel.Info
    };
  }

  async timeLastModified() {
    try {
      const stats = await fs.promises.stat(this.filePath);
      return stats.mtime.getTime();
    } catch (err) {
      if (err.code === Constants.ENOENT_ERROR) {
        // file does not exist, so it's never been modified
        this.logger.verbose("Cache file does not exist");
        return 0;
      }

      throw PersistenceError.createFileSystemError(err.code, err.message);
    }
  }

  async createCacheFile() {
    await this.createFileDirectory(); // File is created only if it does not exist

    const fileHandle = await fs.promises.open(this.filePath, "a");
    await fileHandle.close();
    this.logger.info(`File created at ${this.filePath}`);
  }

  async createFileDirectory() {
    try {
      await fs.promises.mkdir(path.dirname(this.filePath), {
        recursive: true
      });
    } catch (err) {
      if (err.code === Constants.EEXIST_ERROR) {
        this.logger.info(`Directory ${path.dirname(this.filePath)}  already exists`);
      } else {
        throw PersistenceError.createFileSystemError(err.code, err.message);
      }
    }
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

/* eslint-disable-next-line @typescript-eslint/no-var-requires, no-var, import/no-commonjs */
var Dpapi = /*#__PURE__*/require("bindings")({
  bindings: "dpapi",
  userDefinedTries: [["module_root", "node_modules", "@azure", "msal-node-extensions", "build", "Release", "bindings"]]
});

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

(function (DataProtectionScope) {
  DataProtectionScope["CurrentUser"] = "CurrentUser";
  DataProtectionScope["LocalMachine"] = "LocalMachine";
})(exports.DataProtectionScope || (exports.DataProtectionScope = {}));

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Uses CryptProtectData and CryptUnprotectData on Windows to encrypt and decrypt file contents.
 *
 * scope: Scope of the data protection. Either local user or the current machine
 * optionalEntropy: Password or other additional entropy used to encrypt the data
 */

class FilePersistenceWithDataProtection extends BasePersistence {
  constructor(scope, optionalEntropy) {
    super();
    this.scope = scope;
    this.optionalEntropy = optionalEntropy ? Buffer.from(optionalEntropy, "utf-8") : null;
  }

  static async create(fileLocation, scope, optionalEntropy, loggerOptions) {
    const persistence = new FilePersistenceWithDataProtection(scope, optionalEntropy);
    persistence.filePersistence = await FilePersistence.create(fileLocation, loggerOptions);
    return persistence;
  }

  async save(contents) {
    try {
      const encryptedContents = Dpapi.protectData(Buffer.from(contents, "utf-8"), this.optionalEntropy, this.scope.toString());
      await this.filePersistence.saveBuffer(encryptedContents);
    } catch (err) {
      throw PersistenceError.createFilePersistenceWithDPAPIError(err.message);
    }
  }

  async load() {
    try {
      const encryptedContents = await this.filePersistence.loadBuffer();

      if (typeof encryptedContents === "undefined" || !encryptedContents || 0 === encryptedContents.length) {
        this.filePersistence.getLogger().info("Encrypted contents loaded from file were null or empty");
        return null;
      }

      return Dpapi.unprotectData(encryptedContents, this.optionalEntropy, this.scope.toString()).toString();
    } catch (err) {
      throw PersistenceError.createFilePersistenceWithDPAPIError(err.message);
    }
  }

  async delete() {
    return this.filePersistence.delete();
  }

  async reloadNecessary(lastSync) {
    return this.filePersistence.reloadNecessary(lastSync);
  }

  getFilePath() {
    return this.filePersistence.getFilePath();
  }

  getLogger() {
    return this.filePersistence.getLogger();
  }

  createForPersistenceValidation() {
    const testCacheFileLocation = `${path.dirname(this.filePersistence.getFilePath())}/test.cache`;
    return FilePersistenceWithDataProtection.create(testCacheFileLocation, exports.DataProtectionScope.CurrentUser);
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Uses reads and writes passwords to macOS keychain
 *
 * serviceName: Identifier used as key for whatever value is stored
 * accountName: Account under which password should be stored
 */

class KeychainPersistence extends BasePersistence {
  constructor(serviceName, accountName) {
    super();
    this.serviceName = serviceName;
    this.accountName = accountName;
  }

  static async create(fileLocation, serviceName, accountName, loggerOptions) {
    const persistence = new KeychainPersistence(serviceName, accountName);
    persistence.filePersistence = await FilePersistence.create(fileLocation, loggerOptions);
    return persistence;
  }

  async save(contents) {
    try {
      await keytar.setPassword(this.serviceName, this.accountName, contents);
    } catch (err) {
      throw PersistenceError.createKeychainPersistenceError(err.message);
    } // Write dummy data to update file mtime


    await this.filePersistence.save("{}");
  }

  async load() {
    try {
      return await keytar.getPassword(this.serviceName, this.accountName);
    } catch (err) {
      throw PersistenceError.createKeychainPersistenceError(err.message);
    }
  }

  async delete() {
    try {
      await this.filePersistence.delete();
      return await keytar.deletePassword(this.serviceName, this.accountName);
    } catch (err) {
      throw PersistenceError.createKeychainPersistenceError(err.message);
    }
  }

  async reloadNecessary(lastSync) {
    return this.filePersistence.reloadNecessary(lastSync);
  }

  getFilePath() {
    return this.filePersistence.getFilePath();
  }

  getLogger() {
    return this.filePersistence.getLogger();
  }

  createForPersistenceValidation() {
    const testCacheFileLocation = `${path.dirname(this.filePersistence.getFilePath())}/test.cache`;
    return KeychainPersistence.create(testCacheFileLocation, "persistenceValidationServiceName", "persistencValidationAccountName");
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Uses reads and writes passwords to Secret Service API/libsecret. Requires libsecret
 * to be installed.
 *
 * serviceName: Identifier used as key for whatever value is stored
 * accountName: Account under which password should be stored
 */

class LibSecretPersistence extends BasePersistence {
  constructor(serviceName, accountName) {
    super();
    this.serviceName = serviceName;
    this.accountName = accountName;
  }

  static async create(fileLocation, serviceName, accountName, loggerOptions) {
    const persistence = new LibSecretPersistence(serviceName, accountName);
    persistence.filePersistence = await FilePersistence.create(fileLocation, loggerOptions);
    return persistence;
  }

  async save(contents) {
    try {
      await keytar.setPassword(this.serviceName, this.accountName, contents);
    } catch (err) {
      throw PersistenceError.createLibSecretError(err.message);
    } // Write dummy data to update file mtime


    await this.filePersistence.save("{}");
  }

  async load() {
    try {
      return await keytar.getPassword(this.serviceName, this.accountName);
    } catch (err) {
      throw PersistenceError.createLibSecretError(err.message);
    }
  }

  async delete() {
    try {
      await this.filePersistence.delete();
      return await keytar.deletePassword(this.serviceName, this.accountName);
    } catch (err) {
      throw PersistenceError.createLibSecretError(err.message);
    }
  }

  async reloadNecessary(lastSync) {
    return this.filePersistence.reloadNecessary(lastSync);
  }

  getFilePath() {
    return this.filePersistence.getFilePath();
  }

  getLogger() {
    return this.filePersistence.getLogger();
  }

  createForPersistenceValidation() {
    const testCacheFileLocation = `${path.dirname(this.filePersistence.getFilePath())}/test.cache`;
    return LibSecretPersistence.create(testCacheFileLocation, "persistenceValidationServiceName", "persistencValidationAccountName");
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
class Environment {
  static get homeEnvVar() {
    return this.getEnvironmentVariable(Constants.ENVIRONMENT.HOME);
  }

  static get lognameEnvVar() {
    return this.getEnvironmentVariable(Constants.ENVIRONMENT.LOGNAME);
  }

  static get userEnvVar() {
    return this.getEnvironmentVariable(Constants.ENVIRONMENT.USER);
  }

  static get lnameEnvVar() {
    return this.getEnvironmentVariable(Constants.ENVIRONMENT.LNAME);
  }

  static get usernameEnvVar() {
    return this.getEnvironmentVariable(Constants.ENVIRONMENT.USERNAME);
  }

  static getEnvironmentVariable(name) {
    return process.env[name];
  }

  static getEnvironmentPlatform() {
    return process.platform;
  }

  static isWindowsPlatform() {
    return this.getEnvironmentPlatform() === Platform.WINDOWS;
  }

  static isLinuxPlatform() {
    return this.getEnvironmentPlatform() === Platform.LINUX;
  }

  static isMacPlatform() {
    return this.getEnvironmentPlatform() === Platform.MACOS;
  }

  static isLinuxRootUser() {
    return process.getuid() === Constants.LINUX_ROOT_USER_GUID;
  }

  static getUserRootDirectory() {
    return !this.isWindowsPlatform ? this.getUserHomeDirOnUnix() : this.getUserHomeDirOnWindows();
  }

  static getUserHomeDirOnWindows() {
    return this.getEnvironmentVariable(Constants.ENVIRONMENT.LOCAL_APPLICATION_DATA);
  }

  static getUserHomeDirOnUnix() {
    if (this.isWindowsPlatform()) {
      throw PersistenceError.createNotSupportedError("Getting the user home directory for unix is not supported in windows");
    }

    if (!msalCommon.StringUtils.isEmpty(this.homeEnvVar)) {
      return this.homeEnvVar;
    }

    let username = null;

    if (!msalCommon.StringUtils.isEmpty(this.lognameEnvVar)) {
      username = this.lognameEnvVar;
    } else if (!msalCommon.StringUtils.isEmpty(this.userEnvVar)) {
      username = this.userEnvVar;
    } else if (!msalCommon.StringUtils.isEmpty(this.lnameEnvVar)) {
      username = this.lnameEnvVar;
    } else if (!msalCommon.StringUtils.isEmpty(this.usernameEnvVar)) {
      username = this.usernameEnvVar;
    }

    if (this.isMacPlatform()) {
      return !msalCommon.StringUtils.isEmpty(username) ? path__default.join("/Users", username) : null;
    } else if (this.isLinuxPlatform()) {
      if (this.isLinuxRootUser()) {
        return "/root";
      } else {
        return !msalCommon.StringUtils.isEmpty(username) ? path__default.join("/home", username) : null;
      }
    } else {
      throw PersistenceError.createNotSupportedError("Getting the user home directory for unix is not supported in windows");
    }
  }

}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
class PersistenceCreator {
  static async createPersistence(config) {
    let peristence; // On Windows, uses a DPAPI encrypted file

    if (Environment.isWindowsPlatform()) {
      if (!config.cachePath || !config.dataProtectionScope) {
        throw PersistenceError.createPersistenceNotValidatedError("Cache path and/or data protection scope not provided for the FilePersistenceWithDataProtection cache plugin");
      }

      peristence = await FilePersistenceWithDataProtection.create(config.cachePath, exports.DataProtectionScope.CurrentUser);
    } // On Mac, uses keychain.
    else if (Environment.isMacPlatform()) {
        if (!config.cachePath || !config.serviceName || !config.accountName) {
          throw PersistenceError.createPersistenceNotValidatedError("Cache path, service name and/or account name not provided for the KeychainPersistence cache plugin");
        }

        peristence = await KeychainPersistence.create(config.cachePath, config.serviceName, config.accountName);
      } // On Linux, uses  libsecret to store to secret service. Libsecret has to be installed.
      else if (Environment.isLinuxPlatform()) {
          if (!config.cachePath || !config.serviceName || !config.accountName) {
            throw PersistenceError.createPersistenceNotValidatedError("Cache path, service name and/or account name not provided for the LibSecretPersistence cache plugin");
          }

          peristence = await LibSecretPersistence.create(config.cachePath, config.serviceName, config.accountName);
        } else {
          throw PersistenceError.createNotSupportedError("The current environment is not supported by msal-node-extensions yet.");
        } // Initially suppress the error thrown during persistence verification to allow us to fallback to plain text


    const isPersistenceVerified = await peristence.verifyPersistence().catch(() => false);

    if (!isPersistenceVerified) {
      if (Environment.isLinuxPlatform() && config.usePlaintextFileOnLinux) {
        if (!config.cachePath) {
          throw PersistenceError.createPersistenceNotValidatedError("Cache path not provided for the FilePersistence cache plugin");
        }

        peristence = await FilePersistence.create(config.cachePath);
        const isFilePersistenceVerified = await peristence.verifyPersistence();

        if (isFilePersistenceVerified) {
          return peristence;
        }
      }

      throw PersistenceError.createPersistenceNotVerifiedError("Persistence could not be verified");
    }

    return peristence;
  }

}

exports.Environment = Environment;
exports.FilePersistence = FilePersistence;
exports.FilePersistenceWithDataProtection = FilePersistenceWithDataProtection;
exports.KeychainPersistence = KeychainPersistence;
exports.LibSecretPersistence = LibSecretPersistence;
exports.PersistenceCachePlugin = PersistenceCachePlugin;
exports.PersistenceCreator = PersistenceCreator;
//# sourceMappingURL=msal-node-extensions.cjs.development.js.map
